package com.ningdatech.pmapi.common.helper.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ningdatech.pmapi.common.helper.UserInfoHelper;
import com.ningdatech.pmapi.organization.model.entity.DingEmployeeInfo;
import com.ningdatech.pmapi.organization.model.entity.DingOrganization;
import com.ningdatech.pmapi.organization.service.IDingEmployeeInfoService;
import com.ningdatech.pmapi.organization.service.IDingOrganizationService;
import com.ningdatech.pmapi.sys.mapper.RoleMapper;
import com.ningdatech.pmapi.sys.model.entity.Role;
import com.ningdatech.pmapi.sys.model.entity.UserRole;
import com.ningdatech.pmapi.sys.service.IUserRoleService;
import com.ningdatech.pmapi.user.constant.UserAvailableEnum;
import com.ningdatech.pmapi.user.entity.UserInfo;
import com.ningdatech.pmapi.user.entity.enumeration.RoleEnum;
import com.ningdatech.pmapi.user.security.auth.model.UserFullInfoDTO;
import com.ningdatech.pmapi.user.service.IUserInfoService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author liuxinxin
 * @date 2023/2/10 下午4:40
 */
@Component
@RequiredArgsConstructor
public class UserInfoHelperImpl implements UserInfoHelper {


    private final IUserInfoService userInfoService;
    private final IDingEmployeeInfoService iDingEmployeeInfoService;
    private final IDingOrganizationService iDingOrganizationService;
    private final IUserRoleService iUserRoleService;
    private final RoleMapper roleMapper;


    @Override
    public String getOrganizationCode(Long userId) {
        UserFullInfoDTO userFullInfo = getUserFullInfo(userId);
        return userFullInfo.getOrganizationCode();
    }

    @Override
    public UserFullInfoDTO getUserFullInfo(Long userId) {
        UserInfo userInfo = userInfoService.getById(userId);
        if (Objects.isNull(userInfo)) {
            return null;
        }
        // 返回用户全量信息
        return getUserFullInfo(userInfo);
    }

    @Override
    public UserFullInfoDTO getUserFullInfo(UserInfo userInfo) {
        UserFullInfoDTO userFullInfo = new UserFullInfoDTO();
        // 获取浙政钉雇员信息 组织信息
        String employeeCode = userInfo.getEmployeeCode();
        if (StringUtils.isNotBlank(employeeCode)) {
            List<DingEmployeeInfo> dingEmployeeInfoList = iDingEmployeeInfoService
                    .list(Wrappers.lambdaQuery(DingEmployeeInfo.class)
                            .eq(DingEmployeeInfo::getEmployeeCode, employeeCode)
                            .eq(DingEmployeeInfo::getMainJob, "true"));
            DingEmployeeInfo dingEmployeeInfo = dingEmployeeInfoList.get(0);


            // 装配用户任职所在单位
            String empPosUnitCode = dingEmployeeInfo.getEmpPosUnitCode();
            if (StringUtils.isNotBlank(empPosUnitCode)) {
                DingOrganization dingOrganization = iDingOrganizationService.getByOrgCode(empPosUnitCode);
                if (Objects.nonNull(dingOrganization)) {
                    userFullInfo.setEmpPosUnitCode(empPosUnitCode);
                    userFullInfo.setEmpPosUnitName(dingOrganization.getOrganizationName());
                    userFullInfo.setRegionCode(dingOrganization.getDivisionCode());
                }
            }

            // 装配用户所在orgCode
            String organizationCode = dingEmployeeInfo.getOrganizationCode();
            List<DingOrganization> dingOrganizationList = iDingOrganizationService.list(Wrappers
                    .lambdaQuery(DingOrganization.class)
                    .eq(DingOrganization::getOrganizationCode, organizationCode));
            if (CollectionUtil.isNotEmpty(dingOrganizationList)) {
                DingOrganization dingOrganization = dingOrganizationList.get(0);
                userFullInfo.setOrganizationCode(dingOrganization.getOrganizationCode());
                userFullInfo.setOrganizationName(dingOrganization.getOrganizationName());
                // 测试使用
                userFullInfo.setRegionLevel(3);
            }
        }

        List<Role> roleList = new ArrayList<>();
        // 获取用户角色列表信息
        List<UserRole> userRoleList = iUserRoleService
                .list(Wrappers.lambdaQuery(UserRole.class)
                        .eq(UserRole::getUserId, userInfo.getId()));
        if (CollectionUtil.isNotEmpty(userRoleList)) {
            List<Long> roleIdList = userRoleList.stream()
                    .map(UserRole::getRoleId).distinct()
                    .collect(Collectors.toList());
            roleList = roleMapper.selectBatchIds(roleIdList);
        }
        userFullInfo.setUserRoleList(roleList);

        // 装配用户任职所在单位

        // 装配返回
        userFullInfo.setUserId(userInfo.getId());
        userFullInfo.setIdentifier(userInfo.getRealName());
        userFullInfo.setRealName(userInfo.getRealName());
        userFullInfo.setEmployeeCode(employeeCode);
        userFullInfo.setUsername(userInfo.getRealName());
        userFullInfo.setMobile(userInfo.getMobile());
        userFullInfo.setAccountId(userInfo.getAccountId());

        String available = userInfo.getAvailable();
        if (StringUtils.isNotBlank(available)) {
            userFullInfo.setAvailable(UserAvailableEnum.valueOf(available));
        }
        return userFullInfo;
    }

    @Override
    public String getUserName(Long userId) {
        UserFullInfoDTO userFullInfo = getUserFullInfo(userId);
        return userFullInfo.getRealName();
    }

    @Override
    public boolean isSuperOrRegionAdmin(Long userId) {
        UserFullInfoDTO userFullInfo = getUserFullInfo(userId);
        if (CollUtil.isNotEmpty(userFullInfo.getUserRoleList())) {
            for (Role role : userFullInfo.getUserRoleList()) {
                if (RoleEnum.SUPER_ADMIN.name().equals(role.getCode()) ||
                        RoleEnum.REGION_MANAGER.name().equals(role.getCode())) {
                    return Boolean.TRUE;
                }
            }
        }
        return Boolean.FALSE;
    }

    @Override
    public UserFullInfoDTO getUserFullInfoByEmployeeCode(String employeeCode) {
        UserInfo userInfo = userInfoService.getUserInfoByEmployeeCode(employeeCode);
        return getUserFullInfo(userInfo);
    }

    @Override
    public String getUserEmpPosUnitCode(Long userId) {
        UserFullInfoDTO userFullInfo = getUserFullInfo(userId);
        return userFullInfo.getEmpPosUnitCode();
    }
}
